"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = ("XYSymmetricField", "YSymmetricField")

class XYSymmetricField:
    All: XYSymmetricField = None
    Zero: XYSymmetricField = None
    YPositive70Percent: XYSymmetricField = None
    YNegative70Percent: XYSymmetricField = None
    YPositive100Percent: XYSymmetricField = None
    YNegative100Percent: XYSymmetricField = None
    XPositive70Percent: XYSymmetricField = None
    XNegative70Percent: XYSymmetricField = None
    XPositive100Percent: XYSymmetricField = None
    XNegative100Percent: XYSymmetricField = None

class YSymmetricField:
    All: YSymmetricField = None
    Zero: YSymmetricField = None
    Positive70Percent: YSymmetricField = None
    Negative70Percent: YSymmetricField = None
    Positive100Percent: YSymmetricField = None
    Negative100Percent: YSymmetricField = None
