"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = ("NSCSagRemoveOptions", "NSCSagShowAs")

class NSCSagRemoveOptions:
    None_: NSCSagRemoveOptions = None
    BaseROC: NSCSagRemoveOptions = None
    BestFitSphere: NSCSagRemoveOptions = None
    AverageSag: NSCSagRemoveOptions = None
    MinimumSag: NSCSagRemoveOptions = None

class NSCSagShowAs:
    Surface: NSCSagShowAs = None
    Contour: NSCSagShowAs = None
    GreyScale: NSCSagShowAs = None
    InverseGreyScale: NSCSagShowAs = None
    FalseColor: NSCSagShowAs = None
    InverseFalseColor: NSCSagShowAs = None
    X_CrossSection: NSCSagShowAs = None
    Y_CrossSection: NSCSagShowAs = None
