"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = ("FileSource", "KnownFolder", "Operation")

class FileSource:
    Project: FileSource = None
    Root: FileSource = None
    Unknown: FileSource = None

class KnownFolder:
    Project: KnownFolder = None
    ABgData: KnownFolder = None
    BlackBoxes: KnownFolder = None
    Coatings: KnownFolder = None
    Configs: KnownFolder = None
    DllBulkScatter: KnownFolder = None
    DllDiffractive: KnownFolder = None
    DllGradientIndex: KnownFolder = None
    DllObjects: KnownFolder = None
    DllPhysicalOptics: KnownFolder = None
    DllSources: KnownFolder = None
    DllSurfaceScatter: KnownFolder = None
    DllSurfaces: KnownFolder = None
    Extend: KnownFolder = None
    GlassCat: KnownFolder = None
    ImaFiles: KnownFolder = None
    Macros: KnownFolder = None
    MeritFunction: KnownFolder = None
    Miscellaneous: KnownFolder = None
    ObjectsApertures: KnownFolder = None
    ObjectsCadFiles: KnownFolder = None
    ObjectsCreoParametricFiles: KnownFolder = None
    ObjectsGridFiles: KnownFolder = None
    ObjectsInventorFiles: KnownFolder = None
    ObjectsPartDesignerObjects: KnownFolder = None
    ObjectsPhosphorsAndFluorescenceFiles: KnownFolder = None
    ObjectsPolygonObjects: KnownFolder = None
    ObjectsStopFiles: KnownFolder = None
    ObjectsSolidWorksFiles: KnownFolder = None
    ObjectsSourcesEulumdat: KnownFolder = None
    ObjectsSourcesIesna: KnownFolder = None
    ObjectsSourcesRadiantSourceModelFiles: KnownFolder = None
    ObjectsSourcesSourceFiles: KnownFolder = None
    ObjectsSourcesSpectrumFiles: KnownFolder = None
    ObjectsTabulatedObjects: KnownFolder = None
    PopBeamFiles: KnownFolder = None
    Profiles: KnownFolder = None
    ScatterData: KnownFolder = None
    Tolerance: KnownFolder = None
    Udo: KnownFolder = None
    ZosApiExtensions: KnownFolder = None
    ZosApiOperands: KnownFolder = None
    ZosApiUserAnalysis: KnownFolder = None
    DllBirefringence: KnownFolder = None
    Unknown: KnownFolder = None

class Operation:
    None_: Operation = None
    GetFilesInUse: Operation = None
    GetAdditionalFiles: Operation = None
