"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = ("SagittalAberrationComponent", "TangentialAberrationComponent")

class SagittalAberrationComponent:
    Aberration_X: SagittalAberrationComponent = None
    Aberration_Y: SagittalAberrationComponent = None

class TangentialAberrationComponent:
    Aberration_Y: TangentialAberrationComponent = None
    Aberration_X: TangentialAberrationComponent = None
