"""OpticStudio analyses from the Extended Scene category."""

from zospy.analyses.extendedscene.geometric_image_analysis import (
    GeometricImageAnalysis,
    GeometricImageAnalysisSettings,
)

__all__ = ("GeometricImageAnalysis", "GeometricImageAnalysisSettings")
