"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = "PathAnalysisSortType"

class PathAnalysisSortType:
    TotalEndingFlux: PathAnalysisSortType = None
    NumberOfObjectsStruck: PathAnalysisSortType = None
    NumberOfBranches: PathAnalysisSortType = None
    LastObjectStruck: PathAnalysisSortType = None
    NumberOfUniqueObjectsStruct: PathAnalysisSortType = None
    AverageOpticalPathLength: PathAnalysisSortType = None
    NumberOfDiffractionEvents: PathAnalysisSortType = None
