"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = (
    "BackgroundOptions",
    "BrightnessOptions",
    "CameraViewpointOptions",
    "ColorRaysByCrossSectionOptions",
    "ColorRaysByNSCOptions",
    "ColorRaysByOptions",
    "DetectorDisplayModeOptions",
    "DetectorPixelColorOptions",
    "DrawSectionOptions",
    "LineThicknessOptions",
    "NumberSegmentsOptions",
    "OpacityOptions",
    "RayTraceOptions",
    "RealPupilOptions",
)

class BackgroundOptions:
    White: BackgroundOptions = None
    Black: BackgroundOptions = None
    Red: BackgroundOptions = None
    Green: BackgroundOptions = None
    Blue: BackgroundOptions = None
    DarkGreen: BackgroundOptions = None
    DarkBlue: BackgroundOptions = None
    Color01: BackgroundOptions = None
    Color02: BackgroundOptions = None
    Color03: BackgroundOptions = None
    Color04: BackgroundOptions = None
    Color05: BackgroundOptions = None
    Color06: BackgroundOptions = None
    Color07: BackgroundOptions = None
    Color08: BackgroundOptions = None
    Color09: BackgroundOptions = None
    Color10: BackgroundOptions = None
    Color11: BackgroundOptions = None
    Color12: BackgroundOptions = None
    Color13: BackgroundOptions = None
    Color14: BackgroundOptions = None
    Color15: BackgroundOptions = None
    Color16: BackgroundOptions = None
    Color17: BackgroundOptions = None
    Color18: BackgroundOptions = None
    Color19: BackgroundOptions = None
    Color20: BackgroundOptions = None
    Color21: BackgroundOptions = None
    Color22: BackgroundOptions = None
    Color23: BackgroundOptions = None
    Color24: BackgroundOptions = None
    Gradient01: BackgroundOptions = None
    Gradient02: BackgroundOptions = None
    Gradient03: BackgroundOptions = None
    Gradient04: BackgroundOptions = None
    Gradient05: BackgroundOptions = None
    Gradient06: BackgroundOptions = None
    Gradient07: BackgroundOptions = None
    Gradient08: BackgroundOptions = None
    Gradient09: BackgroundOptions = None
    Gradient10: BackgroundOptions = None
    GradientUser: BackgroundOptions = None

class BrightnessOptions:
    P100: BrightnessOptions = None
    P90: BrightnessOptions = None
    P80: BrightnessOptions = None
    P70: BrightnessOptions = None
    P60: BrightnessOptions = None
    P50: BrightnessOptions = None
    P40: BrightnessOptions = None
    P30: BrightnessOptions = None
    P20: BrightnessOptions = None
    P10: BrightnessOptions = None

class CameraViewpointOptions:
    Isometric: CameraViewpointOptions = None
    XY: CameraViewpointOptions = None
    YZ: CameraViewpointOptions = None
    XZ: CameraViewpointOptions = None

class ColorRaysByCrossSectionOptions:
    Fields: ColorRaysByCrossSectionOptions = None
    Waves: ColorRaysByCrossSectionOptions = None
    Wavelength: ColorRaysByCrossSectionOptions = None

class ColorRaysByNSCOptions:
    SourceNumber: ColorRaysByNSCOptions = None
    WaveNumber: ColorRaysByNSCOptions = None
    ConfigNumber: ColorRaysByNSCOptions = None
    Wavelength: ColorRaysByNSCOptions = None
    SegmentNumber: ColorRaysByNSCOptions = None

class ColorRaysByOptions:
    Fields: ColorRaysByOptions = None
    Waves: ColorRaysByOptions = None
    Config: ColorRaysByOptions = None
    Wavelength: ColorRaysByOptions = None

class DetectorDisplayModeOptions:
    Consider: DetectorDisplayModeOptions = None
    GreyScaleFlux: DetectorDisplayModeOptions = None
    InverseGreyScaleFlux: DetectorDisplayModeOptions = None
    FalseColorFlux: DetectorDisplayModeOptions = None
    InverseFalseColorFlux: DetectorDisplayModeOptions = None
    GreyScaleIrradiance: DetectorDisplayModeOptions = None
    InverseGreyScaleIrradiance: DetectorDisplayModeOptions = None
    FalseColorIrradiance: DetectorDisplayModeOptions = None
    InverseFalseColorIrradiance: DetectorDisplayModeOptions = None

class DetectorPixelColorOptions:
    DoNotColorIndividualPoints: DetectorPixelColorOptions = None
    ByRaysOnLayout: DetectorPixelColorOptions = None
    ByLastAnalysis: DetectorPixelColorOptions = None

class DrawSectionOptions:
    P100: DrawSectionOptions = None
    P75: DrawSectionOptions = None
    P50: DrawSectionOptions = None
    P25: DrawSectionOptions = None

class LineThicknessOptions:
    Standard: LineThicknessOptions = None
    Thick: LineThicknessOptions = None
    Thickest: LineThicknessOptions = None
    Thinnest: LineThicknessOptions = None
    Thin: LineThicknessOptions = None

class NumberSegmentsOptions:
    S_8: NumberSegmentsOptions = None
    S_16: NumberSegmentsOptions = None
    S_32: NumberSegmentsOptions = None
    S_64: NumberSegmentsOptions = None
    S_128: NumberSegmentsOptions = None

class OpacityOptions:
    Ignore: OpacityOptions = None
    Consider: OpacityOptions = None
    All50Percent: OpacityOptions = None

class RayTraceOptions:
    UseRays: RayTraceOptions = None
    LightningTraceTrueColor: RayTraceOptions = None
    LightningTraceAvgWavelength: RayTraceOptions = None

class RealPupilOptions:
    Pupils_Off: RealPupilOptions = None
    Pupils_4: RealPupilOptions = None
    Pupils_8: RealPupilOptions = None
    Pupils_16: RealPupilOptions = None
    Pupils_32: RealPupilOptions = None
