"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = "DataType"

class DataType:
    Wavefront: DataType = None
    SpotRadius: DataType = None
    Spot_X: DataType = None
    Spot_Y: DataType = None
    StrehlRation: DataType = None
    PTWavefront: DataType = None
