"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = ("ConvertAsphereTypes", "FindBestAsphereTypes")

class ConvertAsphereTypes:
    EvenAsphere: ConvertAsphereTypes = None
    ExtendedAsphere: ConvertAsphereTypes = None
    QTypeAsphereType0: ConvertAsphereTypes = None
    QTypeAsphereType1: ConvertAsphereTypes = None

class FindBestAsphereTypes:
    Conic: FindBestAsphereTypes = None
    Polynomial_4thOrder: FindBestAsphereTypes = None
    Polynomial_6thOrder: FindBestAsphereTypes = None
    Polynomial_8thOrder: FindBestAsphereTypes = None
    Polynomial_10thOrder: FindBestAsphereTypes = None
    Polynomial_12thOrder: FindBestAsphereTypes = None
