"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = "MultiConfigOperandType"

class MultiConfigOperandType:
    MOFF: MultiConfigOperandType = None
    AICN: MultiConfigOperandType = None
    AFOC: MultiConfigOperandType = None
    APDF: MultiConfigOperandType = None
    APDT: MultiConfigOperandType = None
    APDX: MultiConfigOperandType = None
    APDY: MultiConfigOperandType = None
    APER: MultiConfigOperandType = None
    APMN: MultiConfigOperandType = None
    APMX: MultiConfigOperandType = None
    APTP: MultiConfigOperandType = None
    CADX: MultiConfigOperandType = None
    CADY: MultiConfigOperandType = None
    CAOR: MultiConfigOperandType = None
    CATX: MultiConfigOperandType = None
    CATY: MultiConfigOperandType = None
    CATZ: MultiConfigOperandType = None
    CBDX: MultiConfigOperandType = None
    CBDY: MultiConfigOperandType = None
    CBOR: MultiConfigOperandType = None
    CBTX: MultiConfigOperandType = None
    CBTY: MultiConfigOperandType = None
    CBTZ: MultiConfigOperandType = None
    CONN: MultiConfigOperandType = None
    COTN: MultiConfigOperandType = None
    CPCN: MultiConfigOperandType = None
    CROR: MultiConfigOperandType = None
    CRSR: MultiConfigOperandType = None
    CRVT: MultiConfigOperandType = None
    CSP1: MultiConfigOperandType = None
    CSP2: MultiConfigOperandType = None
    CWGT: MultiConfigOperandType = None
    EDVA: MultiConfigOperandType = None
    FLTP: MultiConfigOperandType = None
    FLWT: MultiConfigOperandType = None
    FVAN: MultiConfigOperandType = None
    FVCX: MultiConfigOperandType = None
    FVCY: MultiConfigOperandType = None
    FVDX: MultiConfigOperandType = None
    FVDY: MultiConfigOperandType = None
    GCRS: MultiConfigOperandType = None
    GLSS: MultiConfigOperandType = None
    GPEX: MultiConfigOperandType = None
    GPEY: MultiConfigOperandType = None
    GPIU: MultiConfigOperandType = None
    GPJX: MultiConfigOperandType = None
    GPJY: MultiConfigOperandType = None
    GPPX: MultiConfigOperandType = None
    GPPY: MultiConfigOperandType = None
    GQPO: MultiConfigOperandType = None
    HOLD: MultiConfigOperandType = None
    IGNM: MultiConfigOperandType = None
    IGNR: MultiConfigOperandType = None
    LTTL: MultiConfigOperandType = None
    MABB: MultiConfigOperandType = None
    MCOM: MultiConfigOperandType = None
    MDPG: MultiConfigOperandType = None
    MIND: MultiConfigOperandType = None
    MTFU: MultiConfigOperandType = None
    NCOM: MultiConfigOperandType = None
    NCOT: MultiConfigOperandType = None
    NGLS: MultiConfigOperandType = None
    NPAR: MultiConfigOperandType = None
    NPOS: MultiConfigOperandType = None
    NPRO: MultiConfigOperandType = None
    PAR1: MultiConfigOperandType = None
    PAR2: MultiConfigOperandType = None
    PAR3: MultiConfigOperandType = None
    PAR4: MultiConfigOperandType = None
    PAR5: MultiConfigOperandType = None
    PAR6: MultiConfigOperandType = None
    PAR7: MultiConfigOperandType = None
    PAR8: MultiConfigOperandType = None
    PRAM: MultiConfigOperandType = None
    PRES: MultiConfigOperandType = None
    PRWV: MultiConfigOperandType = None
    PSCX: MultiConfigOperandType = None
    PSCY: MultiConfigOperandType = None
    PSHX: MultiConfigOperandType = None
    PSHY: MultiConfigOperandType = None
    PSHZ: MultiConfigOperandType = None
    PSP1: MultiConfigOperandType = None
    PSP2: MultiConfigOperandType = None
    PSP3: MultiConfigOperandType = None
    PUCN: MultiConfigOperandType = None
    PXAR: MultiConfigOperandType = None
    RAAM: MultiConfigOperandType = None
    SATP: MultiConfigOperandType = None
    SDIA: MultiConfigOperandType = None
    SDRW: MultiConfigOperandType = None
    STPS: MultiConfigOperandType = None
    SWCN: MultiConfigOperandType = None
    TCEX: MultiConfigOperandType = None
    TELE: MultiConfigOperandType = None
    TEMP: MultiConfigOperandType = None
    THIC: MultiConfigOperandType = None
    TSP1: MultiConfigOperandType = None
    TSP2: MultiConfigOperandType = None
    TSP3: MultiConfigOperandType = None
    UDAF: MultiConfigOperandType = None
    WAVE: MultiConfigOperandType = None
    WLWT: MultiConfigOperandType = None
    XFIE: MultiConfigOperandType = None
    YFIE: MultiConfigOperandType = None
    OPDR: MultiConfigOperandType = None
    SRTS: MultiConfigOperandType = None
    MCHI: MultiConfigOperandType = None
    IGTO: MultiConfigOperandType = None
    FTAN: MultiConfigOperandType = None
    FCMM: MultiConfigOperandType = None
    CHZN: MultiConfigOperandType = None
    MCSD: MultiConfigOperandType = None
    CSDX: MultiConfigOperandType = None
    CSDY: MultiConfigOperandType = None
    CSTX: MultiConfigOperandType = None
    CSTY: MultiConfigOperandType = None
    CSTZ: MultiConfigOperandType = None
