"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = ("OptimizationAlgorithm", "OptimizationCycles", "OptimizationSaveCount")

class OptimizationAlgorithm:
    DampedLeastSquares: OptimizationAlgorithm = None
    OrthogonalDescent: OptimizationAlgorithm = None

class OptimizationCycles:
    Automatic: OptimizationCycles = None
    Fixed_1_Cycle: OptimizationCycles = None
    Fixed_5_Cycles: OptimizationCycles = None
    Fixed_10_Cycles: OptimizationCycles = None
    Fixed_50_Cycles: OptimizationCycles = None
    Infinite: OptimizationCycles = None

class OptimizationSaveCount:
    Save_10: OptimizationSaveCount = None
    Save_20: OptimizationSaveCount = None
    Save_30: OptimizationSaveCount = None
    Save_40: OptimizationSaveCount = None
    Save_50: OptimizationSaveCount = None
    Save_60: OptimizationSaveCount = None
    Save_70: OptimizationSaveCount = None
    Save_80: OptimizationSaveCount = None
    Save_90: OptimizationSaveCount = None
    Save_100: OptimizationSaveCount = None
