"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations
from . import RMSField, RMSFieldMap

__all__ = (
    "RMSField",
    "RMSFieldMap",
    "FieldDensities",
    "FocusDensities",
    "Method",
    "Orientations",
    "RayDensities",
    "ReferTo",
    "WaveDensities",
)

class FieldDensities:
    FieldDens_5: FieldDensities = None
    FieldDens_10: FieldDensities = None
    FieldDens_15: FieldDensities = None
    FieldDens_20: FieldDensities = None
    FieldDens_25: FieldDensities = None
    FieldDens_30: FieldDensities = None
    FieldDens_35: FieldDensities = None
    FieldDens_40: FieldDensities = None
    FieldDens_45: FieldDensities = None
    FieldDens_50: FieldDensities = None
    FieldDens_55: FieldDensities = None
    FieldDens_60: FieldDensities = None
    FieldDens_65: FieldDensities = None
    FieldDens_70: FieldDensities = None
    FieldDens_75: FieldDensities = None
    FieldDens_80: FieldDensities = None
    FieldDens_85: FieldDensities = None
    FieldDens_90: FieldDensities = None
    FieldDens_95: FieldDensities = None
    FieldDens_100: FieldDensities = None

class FocusDensities:
    FocusDens_5: FocusDensities = None
    FocusDens_10: FocusDensities = None
    FocusDens_15: FocusDensities = None
    FocusDens_20: FocusDensities = None
    FocusDens_25: FocusDensities = None
    FocusDens_30: FocusDensities = None
    FocusDens_35: FocusDensities = None
    FocusDens_40: FocusDensities = None
    FocusDens_45: FocusDensities = None
    FocusDens_50: FocusDensities = None
    FocusDens_55: FocusDensities = None
    FocusDens_60: FocusDensities = None
    FocusDens_65: FocusDensities = None
    FocusDens_70: FocusDensities = None
    FocusDens_75: FocusDensities = None
    FocusDens_80: FocusDensities = None
    FocusDens_85: FocusDensities = None
    FocusDens_90: FocusDensities = None
    FocusDens_95: FocusDensities = None
    FocusDens_100: FocusDensities = None

class Method:
    GaussQuad: Method = None
    RectArray: Method = None

class Orientations:
    Plus_Y: Orientations = None
    Minus_Y: Orientations = None
    Plus_X: Orientations = None
    Minus_X: Orientations = None

class RayDensities:
    RayDens_1: RayDensities = None
    RayDens_32x32: RayDensities = None
    RayDens_64x64: RayDensities = None
    RayDens_2: RayDensities = None
    RayDens_128x128: RayDensities = None
    RayDens_3: RayDensities = None
    RayDens_256x256: RayDensities = None
    RayDens_4: RayDensities = None
    RayDens_5: RayDensities = None
    RayDens_512x512: RayDensities = None
    RayDens_1024x1024: RayDensities = None
    RayDens_6: RayDensities = None
    RayDens_2048x2048: RayDensities = None
    RayDens_7: RayDensities = None
    RayDens_4096x4096: RayDensities = None
    RayDens_8: RayDensities = None
    RayDens_8192x8192: RayDensities = None
    RayDens_9: RayDensities = None
    RayDens_10: RayDensities = None
    RayDens_16384x16384: RayDensities = None
    RayDens_11: RayDensities = None
    RayDens_12: RayDensities = None
    RayDens_13: RayDensities = None
    RayDens_14: RayDensities = None
    RayDens_15: RayDensities = None
    RayDens_16: RayDensities = None
    RayDens_17: RayDensities = None
    RayDens_18: RayDensities = None
    RayDens_19: RayDensities = None
    RayDens_20: RayDensities = None

class ReferTo:
    ChiefRay: ReferTo = None
    Centroid: ReferTo = None

class WaveDensities:
    WaveDens_5: WaveDensities = None
    WaveDens_10: WaveDensities = None
    WaveDens_15: WaveDensities = None
    WaveDens_20: WaveDensities = None
    WaveDens_25: WaveDensities = None
    WaveDens_30: WaveDensities = None
    WaveDens_35: WaveDensities = None
    WaveDens_40: WaveDensities = None
    WaveDens_45: WaveDensities = None
    WaveDens_50: WaveDensities = None
    WaveDens_55: WaveDensities = None
    WaveDens_60: WaveDensities = None
    WaveDens_65: WaveDensities = None
    WaveDens_70: WaveDensities = None
    WaveDens_75: WaveDensities = None
    WaveDens_80: WaveDensities = None
    WaveDens_85: WaveDensities = None
    WaveDens_90: WaveDensities = None
    WaveDens_95: WaveDensities = None
    WaveDens_100: WaveDensities = None
