"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = "DataType"

class DataType:
    Wavefront = 0
    SpotRadius = 1
    Spot_X = 2
    Spot_Y = 3
    StrehlRation = 4
