"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = ("EncircledEnergyTypes", "ExtendedSourceTypes", "GeometricLineEdgeSpreadTypes", "ReferToTypes")

class EncircledEnergyTypes:
    Encircled: EncircledEnergyTypes = None
    X_Only: EncircledEnergyTypes = None
    Y_Only: EncircledEnergyTypes = None
    Ensquared: EncircledEnergyTypes = None

class ExtendedSourceTypes:
    Encircled: ExtendedSourceTypes = None
    X_Only: ExtendedSourceTypes = None
    Y_Only: ExtendedSourceTypes = None
    Ensquared: ExtendedSourceTypes = None
    X_Distrib: ExtendedSourceTypes = None
    Y_Distrib: ExtendedSourceTypes = None

class GeometricLineEdgeSpreadTypes:
    LineEdge: GeometricLineEdgeSpreadTypes = None
    Line: GeometricLineEdgeSpreadTypes = None
    Edge: GeometricLineEdgeSpreadTypes = None

class ReferToTypes:
    ChiefRay: ReferToTypes = None
    Centroid: ReferToTypes = None
    Vertex: ReferToTypes = None
