"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = ("MeritColumn", "MeritOperandType")

class MeritColumn:
    Comment: MeritColumn = None
    Param1: MeritColumn = None
    Param2: MeritColumn = None
    Param3: MeritColumn = None
    Param4: MeritColumn = None
    Param5: MeritColumn = None
    Param6: MeritColumn = None
    Param7: MeritColumn = None
    Param8: MeritColumn = None
    Target: MeritColumn = None
    Weight: MeritColumn = None
    Value: MeritColumn = None
    Contrib: MeritColumn = None

class MeritOperandType:
    ACOS: MeritOperandType = None
    ABSO: MeritOperandType = None
    CVVA: MeritOperandType = None
    DENC: MeritOperandType = None
    DENF: MeritOperandType = None
    DIFF: MeritOperandType = None
    AMAG: MeritOperandType = None
    ANAC: MeritOperandType = None
    ANAR: MeritOperandType = None
    ANAX: MeritOperandType = None
    ANAY: MeritOperandType = None
    ANCX: MeritOperandType = None
    ANCY: MeritOperandType = None
    ASIN: MeritOperandType = None
    ASTI: MeritOperandType = None
    ATAN: MeritOperandType = None
    AXCL: MeritOperandType = None
    BIOC: MeritOperandType = None
    CTVA: MeritOperandType = None
    CVGT: MeritOperandType = None
    CVLT: MeritOperandType = None
    CVOL: MeritOperandType = None
    DIMX: MeritOperandType = None
    DISC: MeritOperandType = None
    DISG: MeritOperandType = None
    DIST: MeritOperandType = None
    DIVI: MeritOperandType = None
    DLTN: MeritOperandType = None
    DMFS: MeritOperandType = None
    DMGT: MeritOperandType = None
    DMLT: MeritOperandType = None
    DMVA: MeritOperandType = None
    DXDX: MeritOperandType = None
    DXDY: MeritOperandType = None
    DYDX: MeritOperandType = None
    DYDY: MeritOperandType = None
    EFFL: MeritOperandType = None
    EFLX: MeritOperandType = None
    EFLY: MeritOperandType = None
    ENDX: MeritOperandType = None
    ENPP: MeritOperandType = None
    EPDI: MeritOperandType = None
    EQUA: MeritOperandType = None
    ETGT: MeritOperandType = None
    ETLT: MeritOperandType = None
    ETVA: MeritOperandType = None
    EXPP: MeritOperandType = None
    FCGS: MeritOperandType = None
    FCGT: MeritOperandType = None
    FCUR: MeritOperandType = None
    FICL: MeritOperandType = None
    FICP: MeritOperandType = None
    FOUC: MeritOperandType = None
    GBPD: MeritOperandType = None
    GBPR: MeritOperandType = None
    GBPS: MeritOperandType = None
    GBPW: MeritOperandType = None
    GBPP: MeritOperandType = None
    GBSD: MeritOperandType = None
    P1GT: MeritOperandType = None
    P1LT: MeritOperandType = None
    P1VA: MeritOperandType = None
    P2GT: MeritOperandType = None
    P2LT: MeritOperandType = None
    P2VA: MeritOperandType = None
    P3GT: MeritOperandType = None
    P3LT: MeritOperandType = None
    P3VA: MeritOperandType = None
    P4GT: MeritOperandType = None
    P4LT: MeritOperandType = None
    P4VA: MeritOperandType = None
    P5GT: MeritOperandType = None
    P5LT: MeritOperandType = None
    P5VA: MeritOperandType = None
    P6GT: MeritOperandType = None
    P6LT: MeritOperandType = None
    P6VA: MeritOperandType = None
    P7GT: MeritOperandType = None
    P7LT: MeritOperandType = None
    P7VA: MeritOperandType = None
    P8GT: MeritOperandType = None
    P8LT: MeritOperandType = None
    P8VA: MeritOperandType = None
    PANA: MeritOperandType = None
    PANB: MeritOperandType = None
    PANC: MeritOperandType = None
    PARA: MeritOperandType = None
    PARB: MeritOperandType = None
    PARC: MeritOperandType = None
    PARR: MeritOperandType = None
    PARX: MeritOperandType = None
    PARY: MeritOperandType = None
    PARZ: MeritOperandType = None
    PATX: MeritOperandType = None
    PATY: MeritOperandType = None
    PETC: MeritOperandType = None
    PETZ: MeritOperandType = None
    PIMH: MeritOperandType = None
    PLEN: MeritOperandType = None
    PMAG: MeritOperandType = None
    POWR: MeritOperandType = None
    PRIM: MeritOperandType = None
    PROD: MeritOperandType = None
    PMGT: MeritOperandType = None
    PMLT: MeritOperandType = None
    PMVA: MeritOperandType = None
    POPD: MeritOperandType = None
    QOAC: MeritOperandType = None
    QSUM: MeritOperandType = None
    RAGA: MeritOperandType = None
    RAGB: MeritOperandType = None
    RAGC: MeritOperandType = None
    RAGX: MeritOperandType = None
    RAGY: MeritOperandType = None
    RAGZ: MeritOperandType = None
    RAED: MeritOperandType = None
    RAEN: MeritOperandType = None
    RAID: MeritOperandType = None
    RAIN: MeritOperandType = None
    RANG: MeritOperandType = None
    REAA: MeritOperandType = None
    REAB: MeritOperandType = None
    REAC: MeritOperandType = None
    REAR: MeritOperandType = None
    REAX: MeritOperandType = None
    REAY: MeritOperandType = None
    REAZ: MeritOperandType = None
    RELI: MeritOperandType = None
    GBSR: MeritOperandType = None
    GBSS: MeritOperandType = None
    TGTH: MeritOperandType = None
    TMAS: MeritOperandType = None
    TOTR: MeritOperandType = None
    TRAC: MeritOperandType = None
    TRAD: MeritOperandType = None
    TRAE: MeritOperandType = None
    TRAI: MeritOperandType = None
    TRAN: MeritOperandType = None
    TRAR: MeritOperandType = None
    TRAX: MeritOperandType = None
    TRAY: MeritOperandType = None
    TRCX: MeritOperandType = None
    TRCY: MeritOperandType = None
    TTGT: MeritOperandType = None
    TTHI: MeritOperandType = None
    TTLT: MeritOperandType = None
    GBSW: MeritOperandType = None
    GBSP: MeritOperandType = None
    GCOS: MeritOperandType = None
    GENC: MeritOperandType = None
    GENF: MeritOperandType = None
    GLCA: MeritOperandType = None
    GLCB: MeritOperandType = None
    GLCC: MeritOperandType = None
    GLCX: MeritOperandType = None
    GLCY: MeritOperandType = None
    GLCZ: MeritOperandType = None
    GMTA: MeritOperandType = None
    GMTS: MeritOperandType = None
    GMTT: MeritOperandType = None
    GOTO: MeritOperandType = None
    GPIM: MeritOperandType = None
    GRMN: MeritOperandType = None
    GRMX: MeritOperandType = None
    GTCE: MeritOperandType = None
    HACG: MeritOperandType = None
    HHCN: MeritOperandType = None
    I1GT: MeritOperandType = None
    BIOD: MeritOperandType = None
    BIPF: MeritOperandType = None
    BLNK: MeritOperandType = None
    BSER: MeritOperandType = None
    CENX: MeritOperandType = None
    CENY: MeritOperandType = None
    CMFV: MeritOperandType = None
    CMGT: MeritOperandType = None
    CMLT: MeritOperandType = None
    CMVA: MeritOperandType = None
    CODA: MeritOperandType = None
    COGT: MeritOperandType = None
    COLT: MeritOperandType = None
    COMA: MeritOperandType = None
    CONF: MeritOperandType = None
    CONS: MeritOperandType = None
    COSI: MeritOperandType = None
    COVA: MeritOperandType = None
    CTGT: MeritOperandType = None
    CTLT: MeritOperandType = None
    I2GT: MeritOperandType = None
    I3GT: MeritOperandType = None
    I4GT: MeritOperandType = None
    I5GT: MeritOperandType = None
    I6GT: MeritOperandType = None
    I1LT: MeritOperandType = None
    I2LT: MeritOperandType = None
    I3LT: MeritOperandType = None
    I4LT: MeritOperandType = None
    I5LT: MeritOperandType = None
    I6LT: MeritOperandType = None
    I1VA: MeritOperandType = None
    I2VA: MeritOperandType = None
    I3VA: MeritOperandType = None
    I4VA: MeritOperandType = None
    I5VA: MeritOperandType = None
    I6VA: MeritOperandType = None
    IMAE: MeritOperandType = None
    IMSF: MeritOperandType = None
    INDX: MeritOperandType = None
    ISFN: MeritOperandType = None
    ISNA: MeritOperandType = None
    LACL: MeritOperandType = None
    LINV: MeritOperandType = None
    LOGE: MeritOperandType = None
    LOGT: MeritOperandType = None
    LONA: MeritOperandType = None
    LPTD: MeritOperandType = None
    MAXX: MeritOperandType = None
    MCOG: MeritOperandType = None
    MCOL: MeritOperandType = None
    MCOV: MeritOperandType = None
    MINN: MeritOperandType = None
    MNAB: MeritOperandType = None
    MNCA: MeritOperandType = None
    MNCG: MeritOperandType = None
    MNCT: MeritOperandType = None
    MNCV: MeritOperandType = None
    MNDT: MeritOperandType = None
    MNEA: MeritOperandType = None
    MNEG: MeritOperandType = None
    MNET: MeritOperandType = None
    MNIN: MeritOperandType = None
    MNPD: MeritOperandType = None
    MNSD: MeritOperandType = None
    MSWA: MeritOperandType = None
    MSWS: MeritOperandType = None
    MSWT: MeritOperandType = None
    MTFA: MeritOperandType = None
    MTFS: MeritOperandType = None
    MTFT: MeritOperandType = None
    MXAB: MeritOperandType = None
    MXCA: MeritOperandType = None
    MXCG: MeritOperandType = None
    MXCT: MeritOperandType = None
    MXCV: MeritOperandType = None
    MXDT: MeritOperandType = None
    MXEA: MeritOperandType = None
    MXEG: MeritOperandType = None
    MXET: MeritOperandType = None
    MXIN: MeritOperandType = None
    MXPD: MeritOperandType = None
    MXSD: MeritOperandType = None
    NPGT: MeritOperandType = None
    NPLT: MeritOperandType = None
    NPVA: MeritOperandType = None
    NPXG: MeritOperandType = None
    NPXL: MeritOperandType = None
    NPXV: MeritOperandType = None
    NPYG: MeritOperandType = None
    NPYL: MeritOperandType = None
    NPYV: MeritOperandType = None
    NPZG: MeritOperandType = None
    NPZL: MeritOperandType = None
    NPZV: MeritOperandType = None
    NSDC: MeritOperandType = None
    NSDD: MeritOperandType = None
    NSRA: MeritOperandType = None
    NSTR: MeritOperandType = None
    NTXG: MeritOperandType = None
    NTXL: MeritOperandType = None
    NTXV: MeritOperandType = None
    NTYG: MeritOperandType = None
    NTYL: MeritOperandType = None
    NTYV: MeritOperandType = None
    NTZG: MeritOperandType = None
    NTZL: MeritOperandType = None
    NTZV: MeritOperandType = None
    OSCD: MeritOperandType = None
    OBSN: MeritOperandType = None
    OOFF: MeritOperandType = None
    OPDC: MeritOperandType = None
    OPDM: MeritOperandType = None
    OPDX: MeritOperandType = None
    OPGT: MeritOperandType = None
    OPLT: MeritOperandType = None
    OPVA: MeritOperandType = None
    OPTH: MeritOperandType = None
    OSUM: MeritOperandType = None
    RENA: MeritOperandType = None
    RENB: MeritOperandType = None
    RENC: MeritOperandType = None
    RETX: MeritOperandType = None
    RETY: MeritOperandType = None
    RGLA: MeritOperandType = None
    RSCH: MeritOperandType = None
    RSCE: MeritOperandType = None
    RSRE: MeritOperandType = None
    RSRH: MeritOperandType = None
    RWCH: MeritOperandType = None
    RWCE: MeritOperandType = None
    RWRE: MeritOperandType = None
    RWRH: MeritOperandType = None
    SAGX: MeritOperandType = None
    SAGY: MeritOperandType = None
    SFNO: MeritOperandType = None
    SINE: MeritOperandType = None
    SKIS: MeritOperandType = None
    SKIN: MeritOperandType = None
    SPCH: MeritOperandType = None
    SPHA: MeritOperandType = None
    SQRT: MeritOperandType = None
    SSAG: MeritOperandType = None
    SUMM: MeritOperandType = None
    SVIG: MeritOperandType = None
    TANG: MeritOperandType = None
    TFNO: MeritOperandType = None
    WFNO: MeritOperandType = None
    TTVA: MeritOperandType = None
    UDOP: MeritOperandType = None
    UDOC: MeritOperandType = None
    USYM: MeritOperandType = None
    VOLU: MeritOperandType = None
    WLEN: MeritOperandType = None
    XDVA: MeritOperandType = None
    XDGT: MeritOperandType = None
    XDLT: MeritOperandType = None
    XENC: MeritOperandType = None
    XENF: MeritOperandType = None
    XNEA: MeritOperandType = None
    XNEG: MeritOperandType = None
    XNET: MeritOperandType = None
    XXEA: MeritOperandType = None
    XXEG: MeritOperandType = None
    XXET: MeritOperandType = None
    YNIP: MeritOperandType = None
    ZERN: MeritOperandType = None
    ZPLM: MeritOperandType = None
    ZTHI: MeritOperandType = None
    TOLR: MeritOperandType = None
    FTGT: MeritOperandType = None
    FTLT: MeritOperandType = None
    GLCR: MeritOperandType = None
    EFNO: MeritOperandType = None
    DIVB: MeritOperandType = None
    PROB: MeritOperandType = None
    TCGT: MeritOperandType = None
    TCLT: MeritOperandType = None
    TCVA: MeritOperandType = None
    NORX: MeritOperandType = None
    NORY: MeritOperandType = None
    NORZ: MeritOperandType = None
    NORD: MeritOperandType = None
    COSA: MeritOperandType = None
    MTHA: MeritOperandType = None
    MTHS: MeritOperandType = None
    MTHT: MeritOperandType = None
    BFSD: MeritOperandType = None
    EXPD: MeritOperandType = None
    CVIG: MeritOperandType = None
    GPSX: MeritOperandType = None
    GPSY: MeritOperandType = None
    GPRX: MeritOperandType = None
    GPRY: MeritOperandType = None
    GPRT: MeritOperandType = None
    POWF: MeritOperandType = None
    STHI: MeritOperandType = None
    CNPX: MeritOperandType = None
    CNPY: MeritOperandType = None
    CNAX: MeritOperandType = None
    CNAY: MeritOperandType = None
    STRH: MeritOperandType = None
    CIGT: MeritOperandType = None
    CILT: MeritOperandType = None
    CIVA: MeritOperandType = None
    CEGT: MeritOperandType = None
    CELT: MeritOperandType = None
    CEVA: MeritOperandType = None
    NSST: MeritOperandType = None
    ABCD: MeritOperandType = None
    DISA: MeritOperandType = None
    POWP: MeritOperandType = None
    RECI: MeritOperandType = None
    NSRM: MeritOperandType = None
    FDMO: MeritOperandType = None
    FDRE: MeritOperandType = None
    FREZ: MeritOperandType = None
    NSDE: MeritOperandType = None
    ERFP: MeritOperandType = None
    NSDP: MeritOperandType = None
    POPI: MeritOperandType = None
    ABGT: MeritOperandType = None
    ABLT: MeritOperandType = None
    SMIA: MeritOperandType = None
    SCUR: MeritOperandType = None
    SDRV: MeritOperandType = None
    REVR: MeritOperandType = None
    NSLT: MeritOperandType = None
    NSTW: MeritOperandType = None
    NSRW: MeritOperandType = None
    MNRE: MeritOperandType = None
    MXRE: MeritOperandType = None
    MNRI: MeritOperandType = None
    MXRI: MeritOperandType = None
    CEHX: MeritOperandType = None
    CEHY: MeritOperandType = None
    BLTH: MeritOperandType = None
    GBPZ: MeritOperandType = None
    NSRD: MeritOperandType = None
    NPAF: MeritOperandType = None
    MECS: MeritOperandType = None
    MECT: MeritOperandType = None
    GMTN: MeritOperandType = None
    GMTX: MeritOperandType = None
    MTFN: MeritOperandType = None
    MTFX: MeritOperandType = None
    MTHN: MeritOperandType = None
    MTHX: MeritOperandType = None
    MECA: MeritOperandType = None
    MSWN: MeritOperandType = None
    MSWX: MeritOperandType = None
    GAOI: MeritOperandType = None
    MNAI: MeritOperandType = None
    MXAI: MeritOperandType = None
    HYLD: MeritOperandType = None
    OGSS: MeritOperandType = None
    SPHD: MeritOperandType = None
    SPHS: MeritOperandType = None
    SSLP: MeritOperandType = None
    SCRV: MeritOperandType = None
    DSAG: MeritOperandType = None
    DPHS: MeritOperandType = None
    DSLP: MeritOperandType = None
    DCRV: MeritOperandType = None
    EFLA: MeritOperandType = None
    CARD: MeritOperandType = None
    TSAG: MeritOperandType = None
    PSLP: MeritOperandType = None
    QSLP: MeritOperandType = None
    RRET: MeritOperandType = None
