"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = ("BackgroundColors", "DateTimeType", "EncodingType", "LanguageType", "ShowLineAsType", "ZemaxThemes")

class BackgroundColors:
    Default: BackgroundColors = None
    White: BackgroundColors = None
    LightGray: BackgroundColors = None
    Gray: BackgroundColors = None
    DarkGray: BackgroundColors = None
    Black: BackgroundColors = None

class DateTimeType:
    None_: DateTimeType = None
    Date: DateTimeType = None
    DateTime: DateTimeType = None

class EncodingType:
    ANSI: EncodingType = None
    Unicode: EncodingType = None

class LanguageType:
    Chinese: LanguageType = None
    English: LanguageType = None
    Japanese: LanguageType = None

class ShowLineAsType:
    TextAbove: ShowLineAsType = None
    FileName: ShowLineAsType = None
    ConfigurationNumber: ShowLineAsType = None

class ZemaxThemes:
    Classic: ZemaxThemes = None
    Dark: ZemaxThemes = None
