"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = ("ColorRaysBy", "Patterns", "Reference", "ShowScales")

class ColorRaysBy:
    Fields: ColorRaysBy = None
    Waves: ColorRaysBy = None
    Config: ColorRaysBy = None
    Wavelength: ColorRaysBy = None

class Patterns:
    Square: Patterns = None
    Hexapolar: Patterns = None
    Dithered: Patterns = None

class Reference:
    ChiefRay: Reference = None
    Centroid: Reference = None
    Middle: Reference = None
    Vertex: Reference = None

class ShowScales:
    ScaleBar: ShowScales = None
    Box: ShowScales = None
    Cross: ShowScales = None
    Circle: ShowScales = None
