"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = ("HuygensMtfTypes", "MtfDataTypes", "MtfTypes", "ShowAsTypes", "SurfaceMtfTypes")

class HuygensMtfTypes:
    Modulation: HuygensMtfTypes = None

class MtfDataTypes:
    Average: MtfDataTypes = None
    Tangential: MtfDataTypes = None
    Sagittal: MtfDataTypes = None

class MtfTypes:
    Modulation: MtfTypes = None
    Real: MtfTypes = None
    Imaginary: MtfTypes = None
    Phase: MtfTypes = None
    SquareWave: MtfTypes = None

class ShowAsTypes:
    GreyScale: ShowAsTypes = None
    GreyScaleInverted: ShowAsTypes = None
    FalseColor: ShowAsTypes = None
    FalseColorInverted: ShowAsTypes = None

class SurfaceMtfTypes:
    MTF_Incoherent: SurfaceMtfTypes = None
    Real_Incoherent: SurfaceMtfTypes = None
    Imaginary_Incoherent: SurfaceMtfTypes = None
    MTF_Coherent: SurfaceMtfTypes = None
    Real_Coherent: SurfaceMtfTypes = None
    Imaginary_Coherent: SurfaceMtfTypes = None
