"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = ("DataType", "FoucaultShowAs", "KnifeType", "Types")

class DataType:
    Computed: DataType = None
    Reference: DataType = None
    Difference: DataType = None

class FoucaultShowAs:
    Surface: FoucaultShowAs = None
    Contour: FoucaultShowAs = None
    GreyScale: FoucaultShowAs = None
    InverseGreyScale: FoucaultShowAs = None
    FalseColor: FoucaultShowAs = None
    InverseFalseColor: FoucaultShowAs = None
    X_CrossSection: FoucaultShowAs = None
    Y_CrossSection: FoucaultShowAs = None

class KnifeType:
    Horiz_Above: KnifeType = None
    Horiz_Below: KnifeType = None
    Vert_Right: KnifeType = None
    Vert_Left: KnifeType = None

class Types:
    Linear: Types = None
    Log_Minus_3: Types = None
    Log_Minus_6: Types = None
    Log_Minus_9: Types = None
    Log_Minus_12: Types = None
    Log_Minus_15: Types = None
    Log_Minus_18: Types = None
