"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = ("FftPsfType", "PsfRotation", "PsfSampling")

class FftPsfType:
    Linear: FftPsfType = None
    Log: FftPsfType = None
    Phase: FftPsfType = None
    Real: FftPsfType = None
    Imaginary: FftPsfType = None

class PsfRotation:
    CW0: PsfRotation = None
    CW90: PsfRotation = None
    CW180: PsfRotation = None
    CW270: PsfRotation = None

class PsfSampling:
    PsfS_32x32: PsfSampling = None
    PsfS_64x64: PsfSampling = None
    PsfS_128x128: PsfSampling = None
    PsfS_256x256: PsfSampling = None
    PsfS_512x512: PsfSampling = None
    PsfS_1024x1024: PsfSampling = None
    PsfS_2048x2048: PsfSampling = None
    PsfS_4096x4096: PsfSampling = None
    PsfS_8192x8192: PsfSampling = None
    PsfS_16384x16384: PsfSampling = None
