"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = ("SettingsDataType", "ZemaxColor", "ZemaxOpacity")

class SettingsDataType:
    None_: SettingsDataType = None
    Integer: SettingsDataType = None
    Double: SettingsDataType = None
    Float: SettingsDataType = None
    String: SettingsDataType = None
    Byte: SettingsDataType = None
    Boolean: SettingsDataType = None

class ZemaxColor:
    Default: ZemaxColor = None
    Color1: ZemaxColor = None
    Color2: ZemaxColor = None
    Color3: ZemaxColor = None
    Color4: ZemaxColor = None
    Color5: ZemaxColor = None
    Color6: ZemaxColor = None
    Color7: ZemaxColor = None
    Color8: ZemaxColor = None
    Color9: ZemaxColor = None
    Color10: ZemaxColor = None
    Color11: ZemaxColor = None
    Color12: ZemaxColor = None
    Color13: ZemaxColor = None
    Color14: ZemaxColor = None
    Color15: ZemaxColor = None
    Color16: ZemaxColor = None
    Color17: ZemaxColor = None
    Color18: ZemaxColor = None
    Color19: ZemaxColor = None
    Color20: ZemaxColor = None
    Color21: ZemaxColor = None
    Color22: ZemaxColor = None
    Color23: ZemaxColor = None
    Color24: ZemaxColor = None
    NoColor: ZemaxColor = None

class ZemaxOpacity:
    P100: ZemaxOpacity = None
    P90: ZemaxOpacity = None
    P80: ZemaxOpacity = None
    P70: ZemaxOpacity = None
    P60: ZemaxOpacity = None
    P50: ZemaxOpacity = None
    P40: ZemaxOpacity = None
    P30: ZemaxOpacity = None
    P20: ZemaxOpacity = None
    P10: ZemaxOpacity = None
    P00: ZemaxOpacity = None
