"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = ("ToleranceColumn", "ToleranceOperandType")

class ToleranceColumn:
    Param1: ToleranceColumn = None
    Param2: ToleranceColumn = None
    Param3: ToleranceColumn = None
    Nominal: ToleranceColumn = None
    Min: ToleranceColumn = None
    Max: ToleranceColumn = None
    Comment: ToleranceColumn = None

class ToleranceOperandType:
    TOFF: ToleranceOperandType = None
    TRAD: ToleranceOperandType = None
    TCUR: ToleranceOperandType = None
    TFRN: ToleranceOperandType = None
    TTHI: ToleranceOperandType = None
    TCON: ToleranceOperandType = None
    TSDI: ToleranceOperandType = None
    TSDX: ToleranceOperandType = None
    TSDY: ToleranceOperandType = None
    TSDR: ToleranceOperandType = None
    TSTX: ToleranceOperandType = None
    TSTY: ToleranceOperandType = None
    TIRX: ToleranceOperandType = None
    TIRY: ToleranceOperandType = None
    TIRR: ToleranceOperandType = None
    TPAR: ToleranceOperandType = None
    TEDV: ToleranceOperandType = None
    TEDX: ToleranceOperandType = None
    TEDY: ToleranceOperandType = None
    TEDR: ToleranceOperandType = None
    TETX: ToleranceOperandType = None
    TETY: ToleranceOperandType = None
    TETZ: ToleranceOperandType = None
    TUDX: ToleranceOperandType = None
    TUDY: ToleranceOperandType = None
    TUTX: ToleranceOperandType = None
    TUTY: ToleranceOperandType = None
    TUTZ: ToleranceOperandType = None
    TIND: ToleranceOperandType = None
    TABB: ToleranceOperandType = None
    TEXI: ToleranceOperandType = None
    SAVE: ToleranceOperandType = None
    COMP: ToleranceOperandType = None
    CPAR: ToleranceOperandType = None
    CEDV: ToleranceOperandType = None
    CMCO: ToleranceOperandType = None
    STAT: ToleranceOperandType = None
    TWAV: ToleranceOperandType = None
    TNPS: ToleranceOperandType = None
    TNPA: ToleranceOperandType = None
    TMCO: ToleranceOperandType = None
    TEZI: ToleranceOperandType = None
    TCMU: ToleranceOperandType = None
    SEED: ToleranceOperandType = None
    COMM: ToleranceOperandType = None
    TPAI: ToleranceOperandType = None
    TCIO: ToleranceOperandType = None
    TCEO: ToleranceOperandType = None
    TNMA: ToleranceOperandType = None
    CNPA: ToleranceOperandType = None
    CNPS: ToleranceOperandType = None
    TRLX: ToleranceOperandType = None
    TRLY: ToleranceOperandType = None
    TRLR: ToleranceOperandType = None
    TARX: ToleranceOperandType = None
    TARY: ToleranceOperandType = None
    TARR: ToleranceOperandType = None
    ISOA: ToleranceOperandType = None
    ISOB: ToleranceOperandType = None
    ISOC: ToleranceOperandType = None
    ISOD: ToleranceOperandType = None
    MPVT: ToleranceOperandType = None
