"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

from zospy.api._ZOSAPI.Analysis.Settings import IAS_

__all__ = ("IAS_RTA_Angle", "IAS_RTA_Wavelength")

class IAS_RTA_Angle(IAS_):
    @property
    def Surface(self) -> IAS_Surface: ...
    @property
    def Wavelength(self) -> IAS_Wavelength: ...
    @property
    def ObjectNumber(self) -> int: ...
    @ObjectNumber.setter
    def ObjectNumber(self, value: int) -> None: ...
    @property
    def FaceNumber(self) -> int: ...
    @FaceNumber.setter
    def FaceNumber(self, value: int) -> None: ...
    @property
    def MinAngle(self) -> float: ...
    @MinAngle.setter
    def MinAngle(self, value: float) -> None: ...
    @property
    def MaxAngle(self) -> float: ...
    @MaxAngle.setter
    def MaxAngle(self, value: float) -> None: ...
    @property
    def MinY(self) -> float: ...
    @MinY.setter
    def MinY(self, value: float) -> None: ...
    @property
    def MaxY(self) -> float: ...
    @MaxY.setter
    def MaxY(self, value: float) -> None: ...
    @property
    def ReverseDirection(self) -> bool: ...
    @ReverseDirection.setter
    def ReverseDirection(self, value: bool) -> None: ...

class IAS_RTA_Wavelength(IAS_):
    @property
    def Surface(self) -> IAS_Surface: ...
    @property
    def Angle(self) -> float: ...
    @Angle.setter
    def Angle(self, value: float) -> None: ...
    @property
    def ObjectNumber(self) -> int: ...
    @ObjectNumber.setter
    def ObjectNumber(self, value: int) -> None: ...
    @property
    def FaceNumber(self) -> int: ...
    @FaceNumber.setter
    def FaceNumber(self, value: int) -> None: ...
    @property
    def MinWavelength(self) -> float: ...
    @MinWavelength.setter
    def MinWavelength(self, value: float) -> None: ...
    @property
    def MaxWavelength(self) -> float: ...
    @MaxWavelength.setter
    def MaxWavelength(self, value: float) -> None: ...
    @property
    def MinY(self) -> float: ...
    @MinY.setter
    def MinY(self, value: float) -> None: ...
    @property
    def MaxY(self) -> float: ...
    @MaxY.setter
    def MaxY(self, value: float) -> None: ...
    @property
    def ReverseDirection(self) -> bool: ...
    @ReverseDirection.setter
    def ReverseDirection(self, value: bool) -> None: ...
