"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from enum import Enum

__all__ = "LMXF_Platforms"

class LMXF_Platforms(Enum):
    Unknown = 0
    SolidWorks = 1
    Creo = 2
