"""This file provides autocompletions for the ZOS-API and was automatically generated.
It should not be edited manually.
"""

from __future__ import annotations

__all__ = ("ISAberrationTypes", "ISFlipTypes", "ISSamplings", "ISShowAsTypes")

class ISAberrationTypes:
    None_: ISAberrationTypes = None
    Geometric: ISAberrationTypes = None
    Diffraction: ISAberrationTypes = None

class ISFlipTypes:
    None_: ISFlipTypes = None
    TopBottom: ISFlipTypes = None
    LeftRight: ISFlipTypes = None
    TopBottomLeftRight: ISFlipTypes = None

class ISSamplings:
    None_: ISSamplings = None
    X2: ISSamplings = None
    X4: ISSamplings = None
    X8: ISSamplings = None
    X16: ISSamplings = None
    X32: ISSamplings = None
    X64: ISSamplings = None

class ISShowAsTypes:
    SimulatedImage: ISShowAsTypes = None
    SourceBitmap: ISShowAsTypes = None
    PSFGrid: ISShowAsTypes = None
